#!/usr/bin/python
# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A module for global constants."""

import platform

OSLOGIN_CONTROL_SCRIPT = 'google_oslogin_control'
OSLOGIN_NSS_CACHE_SCRIPT = 'google_oslogin_nss_cache'

if platform.system() == 'FreeBSD':
  BOTOCONFDIR = '/usr/local'
  LOCALBASE = '/usr/local'
  LOCALSTATEDIR = '/var/spool'
  OSLOGIN_NSS_CACHE = '/usr/local/etc/oslogin_passwd.cache'
  SYSCONFDIR = '/usr/local/etc'
  SYSLOG_SOCKET = '/var/run/log'
elif platform.system() == 'OpenBSD':
  BOTOCONFDIR = ''
  LOCALBASE = '/usr/local'
  LOCALSTATEDIR = '/var/spool'
  OSLOGIN_NSS_CACHE = '/usr/local/etc/oslogin_passwd.cache'
  SYSCONFDIR = '/usr/local/etc'
  SYSLOG_SOCKET = '/dev/log'
else:
  BOTOCONFDIR = ''
  LOCALBASE = ''
  LOCALSTATEDIR = '/var'
  OSLOGIN_NSS_CACHE = '/etc/oslogin_passwd.cache'
  SYSCONFDIR = '/etc/default'
  SYSLOG_SOCKET = '/dev/log'
